class HttpBaseDataType:
    WEB_SERVER = "webserver"
    CMS = "cms"
    REQUEST = "request"
    COOKIE = "cookie"
    FORM = "form"
    QUERY = "query"
    JSON = "json"
    HEADER = "header"
    HTTP = "http"
