class HttpBaseDataName:
    METHODE = "methode"
    RAW_URL = "rawurl"
    DEBUG = "debug"
    HOST = "host"
    COOKIE = "cookie"
    CONTENT_TYPE = "content-type"
    FULL_URL = "full-url"
    PORT = "port"
    URL = "url"
    REQUEST_ID = "request-id"
    HOST_IP = "hostip"
    HOST_PORT = "hostport"
    CLIENT_IP = "clientip"
    DATE = "date"
    DATE2 = "date2"
    DATE3 = "date3"
    TIME = "time"
    TIME2 = "time2"
    BODY = "body"
    HTTP = "http"
    WEB_SERVER = "webserver"
    CONTENT = "content"
    BLOB_CONTENT = "blob-content"
    INDEX = "index"
    HEADER_CODE = "headercode"
    FILE_PATH = "filepath"
    MIME = "mime"
    GZIP = "gzip"
